/* FitTracks.java

		Fit tracks.

 Created by  Nick Sinev  May '99   
 Modified by
   Mike Ronan    May '01    - 

 */

package hep.lcd.recon.ztracking;

import hep.lcd.event.*;
import hep.lcd.util.driver.*;

import hep.analysis.*;


/**   Fit tracks. */
public class FitTracks extends AbstractProcessor
{
    boolean FitWhileFind = false;


    public FitTracks(Tracker tracker, VertexDetector vxd, TrackFitter fitter)
    {
	this(tracker,vxd,fitter,false);
    }
    public FitTracks(Tracker tracker, VertexDetector vxd, TrackFitter fitter, boolean hist)
    {
 	System.out.println(ClassName+"  version 1.1");

	this.tracker = tracker;
	this.vxd = vxd;
	this.fitter = fitter;

        setupParameters(tracker,vxd);

	// Testing
	//fitter.useVXD(false);

	if (!hist) setNoHist();
	else setHist();

	NPrintEvents = 5; NDebugEvents = 1;
    }
    /** Turns on Track fitting production histograms. */
    public void setHprod() { Hstd = true; setHist(); }
    /** Turns on Track fitting test histograms. */
    public void setHtest() { Hstd = true; setHist(); }
    /** Turns on Track fitting development histograms. */
    public void setHdev() { setHprod(); }

    /** Sets fitting histogram options */
    protected void setHist()  { Hist = ((Hstd) || Hfitter); }
    /** Turns off all histogram packages */
    protected void setNoHist() { Hist = Hstd = Hfitter = false; }


    /** Setup tracker parameters. */
    public void setupParameters(Tracker tracker, VertexDetector vxd)
    {
	System.out.println(ClassName+" Setting up Track fitter");
	fitter.setupFitter(tracker,vxd);
    }


    public TrackFitter getFitter()
    {
	return fitter;
    }

    public void process(LCDEvent event)
    {
	nEvt++;
	if (first) System.out.println(ClassName+" Fit tracks \t\t\t Evt #"+nEvt);

	int ntrks = tracker.getNTracks();

	if (Hist) {
	    HEvents++;
	    HistogramFolder.setDefaultFolder(defaultFolder);
	    if (Hstd) histogram("NTracks found").fill(ntrks);
	}

	if (debug) System.out.println(ClassName+" Fit "+ntrks+" tracks found by TrackFinder.");

	int nfitted = 0;
	//if (nEvt==1) {
	  //for (int itrk=0; itrk<1; itrk++) {
	for (int itrk=0; itrk<ntrks; itrk++) {
	    if (!tracker.fitSuccess(itrk) && !FitWhileFind) {
		//if (debug) System.out.println("Fit track #"+itrk);
		//double tpar[] = tracker.getTrkParam(itrk);

		fitter.fit(itrk,vxd,tracker);

		if (fitter.fitSuccess()) {
		    if (debug && itrk==0) System.out.println("  success! ");
		    nfitted++;

		    //double ftpar[] = new double[5];
		    //ftpar[0] = fitter.getFittedParams().get_d0();
		    //ftpar[1] = fitter.getFittedParams().get_phi0();
		    //ftpar[2] = fitter.getFittedParams().get_omega();
		    //ftpar[3] = fitter.getFittedParams().get_z0();
		    //ftpar[4] = fitter.getFittedParams().get_s();

		    tracker.updateFitStatus(itrk,fitter);
		}
	    }
	  //}
	}
	if (firstEvents) System.out.println("  Fitted "+nfitted+" out of "+ntrks+" tracks.");

	if (Hist) {
	  if (Hstd)
	    histogram("NTracks fitted").fill(nfitted);
	  HistogramFolder.setDefaultFolder("/");
	}

	first = false;
	if (nEvt>=NPrintEvents) firstEvents = false;

    } // End process


    String ClassName = "FitTracks::"; protected String text;
    protected int nEvt = 0;
    protected int NPrintEvents,NDebugEvents;

    // Inline debugging and analysis
    boolean debug = false;

    // Printing
    protected boolean first = true;
    protected boolean firstEvents = true;

    // Histogramming
    protected String defaultFolder = "/Tracking/Fitting";
    protected int HEvents = 0, MaxHistEvts = 1000, MaxTrkHistEvts = 100;
    protected boolean Hist;
    protected boolean Hstd  = false;  // Set for tuning of tracking package
    protected boolean Hfitter = false;

    Tracker tracker;
    TrackFitter fitter;
    VertexDetector vxd;
}
