/* ReconstructedTrack.java

		Reconstructed track.

 Created by  Mike Ronan  Dec '98
 Modified by
   Nick Sinev  Mar '99    - 

 */

package hep.lcd.recon.ztracking;

import hep.lcd.event.*;

import java.io.Serializable;
import java.util.*;

public class ReconstructedTrack extends FoundTrack
{

    /** Track reconstructor to make tracks using Tracker interface. */
    public ReconstructedTrack(Tracker tracker, int it, TrackExtrapolations extrapolations)
    {
	// Get track parameters from Tracker
	double[] trkpar =  tracker.getParameters(it);

	// Internal Track parameters
	parameters = new TrackParameters(trkpar);

	// Track extrapolations
	this.extrapolations = extrapolations;

	// Get Tracker specifications
	Specifications specifications = tracker.getSpecifications();
	BField = specifications.getBField();

	mcp = tracker.getMCParticle(it);
	mcpFraction = tracker.getMCPFraction(it);
	mcpTrackerFraction = tracker.getMCPTrackerFraction(it);

	fitted = tracker.fitSuccess(it);
	NDF = tracker.getNDF(it);
	Chi2 = tracker.getChi2(it);
	if (fitted)
	    this.errmatrix = tracker.getErrorMatrix(it);
    }

    /** Track constructor to make track ... */
    public ReconstructedTrack(TrackParameters parameters, double BField, TrackExtrapolations extrapolations)
    {
	this(parameters,BField,extrapolations,null);
    }
    public ReconstructedTrack(TrackParameters parameters, double BField, TrackExtrapolations extrapolations, MCParticle mcp)
    {
	super(mcp);

	this.parameters = parameters;
	this.BField = BField;
	this.extrapolations = extrapolations;

	double pt = BField*0.3*.01/Math.abs(parameters.omega);
	px = pt * Math.cos(parameters.phi0);
	py = pt * Math.sin(parameters.phi0);
	pz = pt * parameters.s;
	p = Math.sqrt(px*px + py*py + pz*pz);

	charge = parameters.omega>0 ? +1 : -1;
    }

    /** Used by TrackFitter's */
    public ReconstructedTrack(double BField, TrackParameters parameters)
    {
	this(parameters,BField,new TrackExtrapolations(),null);
    }
}
