/* TrackParameters.java

 Created by  Mike Ronan  Dec '98   
 Modified by

 */

package hep.lcd.recon.ztracking;

import java.io.Serializable;
import java.text.*;

public class TrackParameters implements Serializable
{
    final static int nPar = 5;

    public double d0;  // Public access used by TrackFitter's
    public double phi0;
    public double omega;
    public double z0;
    public double s;


    /** Constructor used in track fitting. */
    public TrackParameters(double[] params)
    {
	this(params[0],params[1],params[2],params[3],params[4]);
    }

    /** Standard constructor used in pattern recognition. */
    public TrackParameters(double d0,double phi0,double omega,double z0,double s)
    {
	this.d0 = d0;
	this.phi0 = phi0;
	this.omega = omega;
	this.z0 = z0;
	this.s = s;
    }

    public double get_d0()    { return d0; }
    public double get_phi0()  { return phi0; }
    public double get_omega() { return omega; }
    public double get_z0()    { return z0; }
    public double get_s()     { return s; }

    public void put_d0(double in)    { d0 = in; }
    public void put_phi0(double in)  { phi0 = in; }
    public void put_omega(double in) { omega = in; }
    public void put_z0(double in)    { z0 = in; }
    public void put_s(double in)     { s = in; }

    public double get_curv()   { return omega; }
    public void put_curv(double in)   { omega = in; }

    public double get_tandip() { return s; }
    public void put_tandip(double in) { s = in; }

    public double[] getParameters()
    {
	double[] result = {d0,phi0,omega,z0,s};
	return result;
    }
    public double getParameter(int i)
    {
      switch (i) {
	case 0: return d0;
	case 1: return phi0;
	case 2: return omega;
	case 3: return z0;
	case 4: return s;
	default: throw new IndexOutOfBoundsException(); 			
      }
    }

    public String toString()
    {
	String string = "d0 = "+df.format(d0)+", phi0 = "+df.format(phi0)+
	    ", omega = "+df.format(omega)+", z0 = "+df.format(z0)+", s = "+df.format(s);
	return string;
    }

    static DecimalFormat df = new DecimalFormat();

    static {
	df.setMaximumFractionDigits(3);
    }

    String ClassName = "TrackParameters::";
}
