/* TrackReconstruction.java

		Find and reconstruct tracks.

 Created by  Mike Ronan  Dec '98   Pick up LCD events within JAS framework and
                                   pass 3D points to pattern recognition software.
 Modified by:

 */

package hep.lcd.recon.ztracking;


/**   Picks up LCD events within JAS framework and passes 3D points to pattern recognition software */
public class TrackReconstruction extends hep.lcd.util.driver.Driver
{
    static boolean Hist  = false;


    /**
     * 
     */
    public TrackReconstruction()
    {
	this(Hist);
    }

    public TrackReconstruction(boolean hist)
    {
	System.out.println("");
	System.out.println("TrackReconstruction  version 1.1"+"\n"+
			   "  Running hep.lcd version "+hep.lcd.Version.getVersion());
	System.out.println(" Track reconstruction using TPC only pattern recognition - ");

	// Finds tracks in main tracker.
	tracker = new hep.lcd.recon.ztracking.tpc.TPCTracker(hist);

	// Assign Vertex Detector hits to tracks found in main tracker.
	vertexDetector = new hep.lcd.recon.ztracking.ccd.CCDVertexDetector(tracker,hist);

	// Set up track fitting
	fitter = new hep.lcd.recon.ztracking.fitters.sld.SLDTrackFitter(hist);

	if (fitter!=null) fitTracks = new FitTracks(tracker,vertexDetector,fitter,hist);

	// Add reconstructed tracks to event
	addTracks = new AddReconstructedTracks(tracker,vertexDetector,hist);

        // Set histogram options
        // ---------------------
        if (hist) {
            tracker.setHprod(); // Turn on Tracker production histogram package
        }
        
	// Add processes
        // -------------
	add(tracker);
	add(vertexDetector);
        if (fitTracks!=null) add(fitTracks);
	add(addTracks);
    }

    /**  The following track reconstruction processes are accessible to derived classes:
     * tracker, vertexDetector, fitter, fitTracks and addTracks.
     */
    protected Tracker tracker;
    protected VertexDetector vertexDetector;
    protected TrackFitter fitter;
    protected FitTracks fitTracks;
    protected AddReconstructedTracks addTracks;
}
