/* TrackingStrategy.java

		List of the parameters used by track finder

 Created by Nick Sinev
 Modified by
   Mike Ronan    May '01    Allow for different number of patterns for each tracking package

 */

package hep.lcd.recon.ztracking;


public class TrackingStrategy
{
    protected final int NDetectors = 2;

    protected int[] MaxLevel = new int[NDetectors];
    protected int[] NLevels = new int[NDetectors];
    protected double[][] MinRadius = new double[NDetectors][];

    protected int[] NPatterns = new int[NDetectors];
    protected int[][][] Triplets = new int[NDetectors][][];
    protected int[][]  NPtsReq = new int[NDetectors][];

    protected double[] rTol = new double[NDetectors];
    protected double[] zTol = new double[NDetectors];


    public TrackingStrategy()
    {
	if (trace) System.out.println(ClassName+"  version 1.1.4");
    }

    public void setDefaultStrategy()
    {
	boolean ldebug = true;
	if (ldebug) System.out.println("Default pattern recognition strategy: ");

	int[] levels = {2,2};
	double[][] radius = {{40.,15.},
			     {20.,8.}};

	int[] patterns = {2,3};
	int[][][] triplets = {{{97,45,4},{25,11,3},{10,5,2}},
			      {{9,7,3},{6,4,2},{5,3,1}}};
	int[][] npnts = {{80,18},
			 {9,5,5}};

	rTol[0] = 5.; zTol[0] = 2.;
	rTol[1] = 2.; zTol[1] = 2.;

	if (ldebug) {
	  for (int idet=0; idet<2; idet++) {
	    text=" For detector"+(idet+1)+"\n"+"  min. radius of curvature at each level = ";
	    for (int i=0;i<levels[idet];i++) {
	      text = text+Double.toString(radius[idet][i]); if (i<(levels[idet]-1)) text = text+", "; }
	    System.out.println(text);

	    text="  "+patterns[idet]+" patterns using triplets ";
	    for (int i=0;i<patterns[idet];i++) {
	      text = text+Integer.toString(triplets[idet][i][0])+","+Integer.toString(triplets[idet][i][1])+","+
		  Integer.toString(triplets[idet][i][2]);
	      if (i<(patterns[idet]-1)) text = text+"; "; }
	    System.out.println(text);
	    text=""; for (int i=0;i<patterns[idet];i++) {
	      text = text+Integer.toString(npnts[idet][i]); if (i<(patterns[idet]-1)) text = text+", "; }
	    System.out.println("  w/ # points req. = "+text);
	    System.out.println("  and vertex tolerances in r,z of "+rTol[idet]+", "+zTol[idet]);
	  }
	}
	setStrategy(levels,patterns,radius,triplets,npnts);
    }


    public void setStrategy(int[] levels, int[] patterns, double[][] radius,
			    int[][][] triplets, int[][] npnts)
    {
	if (debug) System.out.println(ClassName+"setStrategy: ");

	for (int i=0; i<NDetectors; i++) {
	  if (debug) System.out.println(" For detector #"+(i+1));
	  MaxLevel[i] = levels[i]-1;
	  NLevels[i] = levels[i];
	  NPatterns[i] = patterns[i];
 
	  MinRadius[i] = new double[NLevels[i]]; text = "";
	  for (int j=0; j<NLevels[i]; j++) {
	    MinRadius[i][j] = radius[i][j];
	    if (debug) { text=text+Double.toString(MinRadius[i][j]); if (j<(NLevels[i]-1)) text = text+", "; }
	  }
	  if (debug) System.out.println("  "+NLevels[i]+" levels of pattern recognition with min. radius of curvature = "+text); text = "";

	  Triplets[i] = new int[NPatterns[i]][3];
	  NPtsReq[i]  = new int[NPatterns[i]];

	  if (debug) text = "  "+NPatterns[i]+" patterns using triplets ";
	  for (int j=0; j<NPatterns[i]; j++) {
	    for(int n=0; n<3; n++) {
	      Triplets[i][j][n] = triplets[i][j][n];
	      if (debug) { text+=Integer.toString(triplets[i][j][n]); if (n<2) text+=","; }
	    }
	    if (debug) { if (j<NPatterns[i]-1) text+="; "; }
	  }
	  if (debug) System.out.println(text);

	  if (debug) text = "  w/ # points req. = ";
	  for (int j=0; j<NPatterns[i]; j++) {
	    NPtsReq[i][j] = npnts[i][j];
	    if (debug) { text+=Integer.toString(NPtsReq[i][j]);
	                 if (j<NPatterns[i]-1) text+=", "; }
	  }
	  if (debug) System.out.println(text);

	  if (debug) System.out.println("  and vertex tolerances in r,z of "+rTol[i]+", "+zTol[i]);
	}
    }


    public int getMaxLevel(int det)
    {
	return MaxLevel[det-1];
    }
    public int getNLevels(int det)
    {
	return NLevels[det-1];
    }
    public double[] getMinRadius(int det)
    {
	return MinRadius[det-1]; 
    }

    public int getNPatterns(int det)
    {
	return NPatterns[det-1];
    }
    public int[][] getTriplets(int det)
    {
	return Triplets[det-1];
    }
    public int[] getNPtsReq(int det)
    {
	return NPtsReq[det-1];
    }

    public double getRTol(int det)
    {
	return rTol[det-1];
    }
 
    public double getZTol(int det)
    {
	return zTol[det-1];
    }

    String ClassName = "TrackingStrategy::";
    protected String text;
    protected boolean debug = false;
    protected boolean trace = false;
}
