/* TrkHitOnTrk.java

 Created by  Mike Ronan  Dec '98   
 Modified by

 */

package hep.lcd.recon.ztracking;

import hep.lcd.event.*;

import java.text.*;


public class TrkHitOnTrk
{
    public String type = "Unknown";

    public double x;
    public double y;
    public double z;
    public double rPhiRms;
    public double zRms;
    public MCParticle mcp=null;

    public double chiSq=0.;
    public double sAng =0.;

    public boolean isActive = true;
    public boolean isUsable = true;
    public boolean mustUse  = false;

    public static boolean firstUpdate = true;
    public static boolean firstFitStuff1 = true;
    public static boolean firstFitStuff2 = true;


    public TrkHitOnTrk(String type,double[] point,double rPhiRms,double zRms)
    {
	this();
	this.type = type;
	this.x = point[0];
	this.y = point[1];
	this.z = point[2];
	this.rPhiRms = rPhiRms;
	this.zRms = zRms;
    }
    public TrkHitOnTrk(String type,double x,double y,double z,double rPhiRms,double zRms)
    {
	this();
	this.type = type;
	this.x = x;
	this.y = y;
	this.z = z;
	this.rPhiRms = rPhiRms;
	this.zRms = zRms;
    }
    public TrkHitOnTrk(String type,double[] point,double rPhiRms,double zRms,MCParticle mcp)
    {
	this(type,point,rPhiRms,zRms);
	this.mcp = mcp;
    }
    public TrkHitOnTrk(String type,double x,double y,double z,double rPhiRms,double zRms,
		       MCParticle mcp)
    {
        this(type,x,y,z,rPhiRms,zRms);
	this.mcp = mcp;
    }
    public TrkHitOnTrk()
    {
	df.setMaximumFractionDigits(4);
    }


    /** Maintained for backward compatibility. */
    public TrkHitOnTrk(String type,double x,double y,double z)
    {
	this();
	this.type = type;
	this.x = x;
	this.y = y;
	this.z = z;
    }
    public TrkHitOnTrk(String type,double[] point,MCParticle mcp)
    {
	this();
	this.type = type;
	this.x = point[0];
	this.y = point[1];
	this.z = point[2];
	this.mcp = mcp;
    }
    public TrkHitOnTrk(String type,double x,double y,double z,MCParticle mcp)
    {
	this();
	this.type = type;
	this.x = x;
	this.y = y;
	this.z = z;
        this.mcp = mcp;
    }

    public String getType() { return type; }
    public boolean trkHit() { return true; }
    public boolean vtxHit() { return false; }
    public void setActive(boolean active) { this.isActive = active; }

    public double getChiSq() { return chiSq; }
    public double getSAng()  { return sAng; }

    public void putChiSq(double chiSq) { this.chiSq = chiSq; }
    public void putSAng(double sAng)   { this.sAng = sAng; }

    public void updateMeasurement()
    {
	if (firstUpdate) {
	    System.out.println(" Updating hit measurement");
	    firstUpdate = false;
	}
    }

    public double updateFitStuff(double[] derivs)
    {
	if (firstFitStuff1) {
	    System.out.println("HelixFitter::updateFitStuff1: ");
	    firstFitStuff1 = false;
	}
	double deltaChiNew = 0.;
	return deltaChiNew;
    }
    public double updateFitStuff()
    {
	if (firstFitStuff2) {
	    System.out.println("HelixFitter::updateFitStuff2: ");
	    firstFitStuff2 = false;
	}
	double deltaChiNew = 0.;
	return deltaChiNew;
    }

    public void setUsability(boolean usability)
    {
	isUsable = usability;
	if (isActive && !isUsable)
	    isActive = false;
	if (!isActive && mustUse)
	    isActive = true;
    }
    public double weight()
    {
	return 1. / Math.sqrt(rPhiRms*rPhiRms + zRms*zRms);
    }

    public MCParticle getMCParticle()
    {
	if (mcp==null && reportAsErrors) {
	    System.err.println("TrkHitOnTrk::getMCParticle: No MCParticle found.");
	    reportAsErrors = false;
	}
	return mcp;
    }

    public void print()
    {
	System.out.println(" "+getType()+" HOT: active: "+isActive+
			   "  x,y,z = "+df.format(x)+", "+df.format(y)+", "+df.format(z));
    }
    public void print(int i)
    {
	System.out.println(" "+getType()+" HOT #"+i+": active: "+isActive+
			   "  x,y,z = "+df.format(x)+", "+df.format(y)+", "+df.format(z));
    }


    private DecimalFormat df = new DecimalFormat();

    static boolean reportErrors   = true;
    static boolean reportAsErrors = true;

}
