/* VertexDetector.java

		Find and reconstruct tracks in vertex detector.

 Created by  Mike Ronan  Dec '98   Assign CCD detector points to tracks.
 Modified by

 */

package hep.lcd.recon.ztracking;

import hep.lcd.util.driver.*;
import hep.lcd.event.*;


/**   Assign CCD detector points to tracks. */
public interface VertexDetector extends Processor
{
    /** Get type of vertex detector. */
    public String getType();

    /** Get Vertex Detector specifications. */
    public Specifications getSpecifications();

    /** Get number of track found. */
    //public int getNTracks();

    /** Get number of points on a track. */
    public int getNPoints(int itrk);
    /** Get a space point on the track. */
    public double[] getTrackPoint(int itrk, int ipnt);
    /** Get the vertex detector hit for a point on the track. */
    public TrackerHit getTrackHit(int itrk, int ipnt);

    /** The Hit ID is unique for any point in tracking system  */
    public int getTrackHitID(int itrk, int ipnt);
    
    /** Start Vertex detector event processing. */
    public void startEvent(LCDEvent e);

    /** Finish Vertex detector event processing. */
    public void finishEvent();

    //public void releaseHits(int it);

    /** Turns on Vertex Detector production histograms. */
    public void setHprod();
    /** Turns on Vertex Detector test histograms. */
    public void setHtest();
    /** Turns on Vertex Detector development histograms. */
    public void setHdev();

    /** Turns on Vertex Detector point histograms: track assignment (xy dev.,z res.,...) */
    public void setHpnts();

    /** Turns off all histogram packages */
    public void setNoHist();
}
