/* CCDVertexDetector.java

		Assign CCD detector points to reconstructed tracks.

 Created by  Mike Ronan  Dec '98   Assign CCD detector points to tracks.
 Modified by:

 */

package hep.lcd.recon.ztracking.ccd;

import hep.lcd.event.*;
import hep.lcd.recon.*;
import hep.lcd.recon.ztracking.*;
import hep.lcd.util.driver.*;

import hep.analysis.*;
import hep.physics.*;

import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.Enumeration;
import java.util.Random;


/**   Assign CCD detector points to tracks. */
final public class CCDVertexDetector extends AbstractVertexDetector
{

    double Sang[] = new double[1+MaxTrkPts];
    double Chis[] = new double[1+MaxTrkPts];
    double ChisN[] = new double[1+MaxTrkPts];

    double ChiSq[] = new double[MaxTracks];

    double dmas = 0.05;          // phi tolerance (cm)
    //double Dzas = 2.0;           // z tolerance (cm)
    double Dzas = 1.0;           // z tolerance (cm)
    // double dmas = 0.5;           // phi tolerance (cm)
    // double Dzas = 0.5;           // z tolerance (cm)


    public CCDVertexDetector(Tracker tracker)
    {
	this(tracker,false);
    }

    public CCDVertexDetector(Tracker tracker, boolean hist)
    {
	super(tracker,hist); // Sets defaultFolder based on Type

	System.out.println(ClassName+"  version 1.1.8.7");
	System.out.println(" Assign CCD hits to tracks from main Tracker - ");

	specifications = new CCDSpecifications();
	NLayers = specifications.getNLayers();

	NLevels = 2;
	SimulateMeasurementError = false;

	if (debug) {
	  System.out.println("  NLayers, NLevels = "+NLayers+", "+NLevels+" (MinLevel = "+MinLevel+")");
	  if (!SimulateMeasurementError)
	      System.out.println("  Note: No measurement resolution smearing in Vertex Detector reconstruction.");
	}
    }


    /** Get type of vertex detector. */
    public String getType() { return "CCD"; }

    /** Set detector r-phi resolution. */
    public void setRPhiResolution(double resolution)
    {
	specifications.setRPhiResolution(resolution);
    }
    /** Set detector z resolution. */
    public void setZResolution(double resolution)
    {
	specifications.setZResolution(resolution);
    }


    /** Get number of points on a track. */
    public int getNPoints(int itrk)
    {
	//if (debug) System.out.println(ClassName+"getNPoints: ");
	if (!processed) assignHits(itrk);
	// if (!processed && VXDIncluded???) assignHits(itrk);
        return ipnt[itrk][0];
    }


/*
    public void startEvent(LCDEvent event)
    {
	nEvt++;
	if (first) { text = ClassName+" Start event in vertex detector \t\t Event #"+nEvt; System.out.println(text); }
	processed = false;
	specifications.update(event);
	// extractHits(event);
	NVxdPnts = event.getVXDHits().getNHits();
    }
*/
    public void process(LCDEvent event)
    {
	nEvt++;
	if (firstEvents) { text = ClassName+" Associate hits in Vertex Detector \t\t Event #"+nEvt; System.out.println(text); }

	// Get Vertex detector hits
	TrackerHits vxdhits = event.getVXDHits();

	processed = false;

	// Vertex detector hit storage
	if (x==null) {
	  x = new double[MaxPoints+1];
	  y = new double[MaxPoints+1];
	  z = new double[MaxPoints+1];
	  KFlag = new boolean[MaxPoints+1];
	  vxdHit = new TrackerHit[MaxPoints+1];

	  ipnt = new int[MaxTracks][1+MaxTrkPts];
	  for (int i=0;i<MaxTracks;i++) ipnt[i][0]=0;
	}

	if (Hist) {
	  HEvents++;
	  HistogramFolder.setDefaultFolder(defaultFolder);
	  if (Hevts) histogram("NHits/evt").fill(vxdhits.getNHits());
	}

	int nvcl=0;
	int nlayer[] = new int[NLayers];
	for (int i=0;i<NLayers;i++) { nlayer[i]=0; }

	for (Enumeration hits = vxdhits.getHits();(hits.hasMoreElements() && nvcl<MaxPoints);) {
	    TrackerHit hit = (TrackerHit) hits.nextElement();
	    //int layer = hit.getLayer() & 0x00000007;
	    int layer = hit.getLayer();
	    if (Hhits) histogram("Hits/layer").fill(layer);
	    nlayer[layer]++;
	    nvcl++;

	    if (printPoints && nvcl <= 10) {
		System.out.println(" Hit #"+nvcl+": layer,n = "+layer+" "+nlayer[layer]);
		// double point[] = hit.getPoint();
		// System.out.println("   x,y,z= "+
		// 		   point[0]+", "+point[1]+", "+point[2]);
		// System.out.println("   r,z= "+
		// 		   Math.sqrt(point[0]*point[0]+point[1]*point[1])+", "+point[2]);
	    }
	}
	if (printPoints && nvcl>10) System.out.println(" ...");
	if (first) {
	  text = " Total of "+nvcl+" Vertex Detector hits.";
	  System.out.println(text);
	  //if (XReconstruction.getBatchMode()) System.err.println(text);
	}

	int nPoints=0;
	int ipoi[] = new int[NLayers];
	int ilon[] = new int[NLayers];
	
	ipoi[0] = ilon[0] = 0;
	for (int layer=1; layer<NLayers; layer++)
	{   ipoi[layer] = ipoi[layer-1]+nlayer[layer-1];
	    ilon[layer] = 0;
	}

	if (SimulateMeasurementError) {
	  double RPhiResolution = specifications.getRPhiResolution();
	  double ZResolution    = specifications.getZResolution();

	  if (first) {
	      System.out.println(" Get Vertex Detector points and simulate measurement errors.");
	      System.out.println("  Measurement resolutions: rphi = "+RPhiResolution+", z = "+ZResolution);
	  }
	  for (Enumeration hits = vxdhits.getHits();(hits.hasMoreElements() && nPoints<MaxPoints);) {
	    TrackerHit hit = (TrackerHit) hits.nextElement();
	    // int layer = hit.getLayer();
	    int layer = hit.getLayer() & 0x00000007;
	    double point[] = hit.getPoint();
	    int iptr = ipoi[layer]+ilon[layer];
	    double r = Math.sqrt(point[0]*point[0] + point[1]*point[1]);
	    double phi = Math.atan2(point[1],point[0]);
	    double RPhi = r*phi + random.nextGaussian()*RPhiResolution;
	    phi = RPhi / r;
	    x[iptr] = r * Math.cos(phi);  y[iptr] = r * Math.sin(phi);
	    z[iptr] = point[2] + random.nextGaussian()*ZResolution;
	    KFlag[iptr] = false;
	    vxdHit[iptr] = hit;
	    ilon[layer]++;
	    nPoints++;
	  }
	} else {
	  if (first)
	    System.out.println("  Note: No measurement resolution smearing in Vertex Detector reconstruction.");

	  if (first) System.out.println(" Get Vertex Detector points");
	  for (Enumeration hits = vxdhits.getHits();(hits.hasMoreElements() && nPoints<MaxPoints);) {
	    TrackerHit hit = (TrackerHit) hits.nextElement();
	    // int layer = hit.getLayer();
	    int layer = hit.getLayer() & 0x00000007;
	    double point[] = hit.getPoint();
	    int iptr = ipoi[layer]+ilon[layer];
	    x[iptr] = point[0];  y[iptr] = point[1];  z[iptr] = point[2];
	    KFlag[iptr] = false;
	    vxdHit[iptr] = hit;
	    ilon[layer]++;
	    nPoints++;
	  }
	}

	// Get number of tracks found by main tracker
	int NTracks = tracker.getNTracks();

	TrackFinder finder = tracker.getFinder();

	// Associate CCD hits at different levels
	if (first) System.out.println(" Associate CCD hits at "+NLevels+" levels.");
	for (int level=MinLevel; level<NLevels; level++) {
	    if (first) System.out.println(" Assign hits to tracks at level "+level);
	    double Dmasu = (double)(1 + level) * dmas;
	    double Ddzax = (double)(1 + level) * Dzas;
	    for (int it=0;it<NTracks;it++) {
		if (level==MinLevel) ipnt[it][0] = 0;
		if (Hpnts && HEvents<MaxPntHistEvts && level==MinLevel) finder.setHpnts();

		int NPtEff = tracker.assignPoints(it,x,y,z,KFlag,NLayers,ipoi,ilon,
						  Ddzax,Dmasu,MaxTrkPts,
						  ipnt,Sang,Chis,ChisN);

		if (Hpnts && HEvents<MaxPntHistEvts) finder.setNoHist();
		if (first) {
		    if (it<MaxPrtTrk) System.out.println(" Number of CCD points on track = "+ipnt[it][0]);
		    if (it==MaxPrtTrk) System.out.println(" ...");
		}
		if (level==MinLevel) {
		    if (Htrks) histogram("NPoints/track").fill(ipnt[it][0]);
		    if (Htrks) histogram("Min. NPtEff/trk").fill(NPtEff);
		}
		if (level==(NLevels-1)) {
		    if ((Hstd||Hassn) && HEvents<=MaxTrkHistEvts) histogram("# VXD points/trk").fill(ipnt[it][0]);

		    //define chis_tk for this track
		    ChiSq[it] = 0;
		    for (int i=1;i<=ipnt[it][0];i++) {
			ChiSq[it] = ChiSq[it] + Chis[i];
		    }
		    if (Htrks) histogram("Chis/trk").fill(ChiSq[it]);
		}
	    }
	}
	if (Hist) {
	  if (NTracks==0 && HEvents<=MaxHistEvts) histogram("Events w/ no tracks").fill(nEvt);
	  HistogramFolder.setDefaultFolder("/");
	}

	processed = true;

	first = false;
	// if (nEvt>=NDebugEvents)
	if (nEvt>=NPrintEvents)
	    firstEvents = printPoints = false;
    } // End process



    String ClassName = "CCDVertexDetector::";

}
